package frauddetection;

import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.walkthrough.common.entity.Alert;
import org.apache.flink.walkthrough.common.entity.Transaction;
import org.apache.flink.walkthrough.common.source.TransactionSource;

/*
 * The fraud detector should output an alert for any account that makes a small transaction
 * immediately followed by a large one.
 * Where small is anything less than $1.00 and large is more than $500.
 *
 * This example is an excerpt of the walk-through example of Apache Flink.
 * Read more:
 * https://nightlies.apache.org/flink/flink-docs-master/docs/try-flink/datastream/
 */
public class FraudDetectionJob {

    public static void main(String[] args) throws Exception {
        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        // Checkpointing
        // Start a checkpoint every 1000 ms
        env.enableCheckpointing(1000);
        env.getCheckpointConfig().setTolerableCheckpointFailureNumber(2);
        env.getCheckpointConfig().setMaxConcurrentCheckpoints(1);
        env.getCheckpointConfig().setCheckpointStorage("file:///tmp/frauddetection/checkpoint");

        /*
         * TransactionSource generates transactions in the form:
         * Transaction{accountId=3, timestamp=1546323120000, amount=0.8}
         * Transaction{accountId=5, timestamp=1546323840000, amount=127.55}
         * Transaction{accountId=3, timestamp=1546324920000, amount=871.15}
         *
         * A fraud occurs when a transaction with a large amount follows
         * a transaction with a small amount within 1 minute. In the previous
         * example, a fraud is occurring at accountId 3.
         */
        DataStream<Transaction> transactions = env
                .addSource(new TransactionSource())
                .name("transactions");

        transactions.print();
        DataStream<Alert> alerts = transactions
                .keyBy(Transaction::getAccountId)
                .process(new FraudDetector())
                .name("fraud-detector");

        alerts.print();

        env.execute("Fraud Detection");
    }
}
